﻿using System;
using System.IO;
using System.Windows;
using System.Windows.Threading;
using System.Diagnostics;

namespace vJoySteering
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        Config config;
        VJoyFeeder feeder;
        DispatcherTimer timerOutput;
        

        public MainWindow()
        {
            InitializeComponent();

            config = new Config();
            try {
                if (File.Exists("config.xml"))
                    config = Config.CreateFromFile("config.xml");
            } catch(Exception)
            {
                MessageBox.Show("ERROR! Couldn't load config!");
            }
            LoadSettingsFromConfig();

            feeder = new VJoyFeeder(config);

            timerOutput = new DispatcherTimer();
            timerOutput.Interval = new TimeSpan(0, 0, 0, 0, 20);
            timerOutput.Tick += TimerOutput_Tick;

            buttonStart_Click(this, null);

            try {
                Process.GetCurrentProcess().PriorityClass = System.Diagnostics.ProcessPriorityClass.High;
            }
            catch (Exception)
            {
                MessageBox.Show("Warning! Can't change process priority.");
            }

        }

        private void TimerOutput_Tick(object sender, EventArgs e)
        {
            if (config.ShowOutput)
            {
                sliderOutput.Value = feeder.Output;
            }
        }

        private void buttonStart_Click(object sender, RoutedEventArgs e)
        {
            feeder.Start();
            timerOutput.Start();
            GC.Collect();
            System.Runtime.GCSettings.LatencyMode = System.Runtime.GCLatencyMode.LowLatency;
            buttonStart.IsEnabled = false;
            buttonStop.IsEnabled = true;
        }

        private void buttonStop_Click(object sender, RoutedEventArgs e)
        {
            StopAll();
            buttonStart.IsEnabled = true;
            buttonStop.IsEnabled = false;
        }

        private void StopAll()
        {
            timerOutput.Stop();
            feeder.Stop();
            System.Runtime.GCSettings.LatencyMode = System.Runtime.GCLatencyMode.Interactive;
            GC.Collect();
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            StopAll();
            try {
                config.Write("config.xml");
            } catch(Exception ex)
            {
                MessageBox.Show("ERROR! Couldn't write config! " + ex.Message);
            }
        }


        private void LoadSettingsFromConfig()
        {
            comboBoxController.SelectedIndex = config.Controller - 1;
            sliderSteeringGamma.Value = config.SteeringGamma;
            sliderSteeringLimit.Value = config.SteeringLockLimit;
            sliderSteeringDeadzone.Value = config.SteeringDeadzone;
            checkBoxShowOutput.IsChecked = config.ShowOutput;
        }

        private void SettingsChanged()
        {
            if (!IsLoaded)
                return;
            config.Controller = comboBoxController.SelectedIndex + 1;
            config.SteeringGamma = sliderSteeringGamma.Value;
            config.SteeringLockLimit = sliderSteeringLimit.Value;
            config.SteeringDeadzone = sliderSteeringDeadzone.Value;
            config.ShowOutput = (bool)checkBoxShowOutput.IsChecked;
            feeder.setupModeLeft = (bool)checkBoxSetupModeLeft.IsChecked;
            feeder.setupModeRight = (bool)checkBoxSetupModeRight.IsChecked;
        }

        private void checkBoxChecked(object sender, RoutedEventArgs e)
        {
            SettingsChanged();
        }

        private void sliderValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            SettingsChanged();
        }

        private void comboBoxSelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            SettingsChanged();
        }
    }
}
