﻿using System;
using System.Threading;
using SharpDX.XInput;
using vJoyInterfaceWrap;


namespace vJoySteering
{

    class VJoyFeeder
    {

        //
        private Thread _thread;
        private readonly object _lock;
        private bool _running;
        public static int SLEEPTIME = 5;
        private Config config;

        // vJoy
        private readonly vJoy joystick;
        private vJoy.JoystickState joystickState;
        private readonly uint joystickIndex;
        private double vJoyMax;
        private double vJoyCenter;
        private double vJoyMin;


        private Controller[] controllers;

        private double output;

        public bool setupModeLeft;
        public bool setupModeRight;


        public VJoyFeeder(Config config)
        {
            this.config = config;
            this._lock = new object();
            controllers = new Controller[4];
            controllers[0] = new Controller(UserIndex.One);
            controllers[1] = new Controller(UserIndex.Two);
            controllers[2] = new Controller(UserIndex.Three);
            controllers[3] = new Controller(UserIndex.Four);
            output = 0;

            joystickIndex = 1;
            joystick = new vJoy();
        }

        public double Output
        {
            get
            {
                return output;
            }
        }

        public Config Config
        {
            set
            {
                config = value;
            }
            get
            {
                return config;
            }
        }


        private void ResetJoystickState()
        {
            joystickState.AxisX = (int)vJoyCenter;
            joystickState.AxisY = (int)vJoyCenter;
            joystickState.AxisZ = (int)vJoyCenter;
            joystickState.AxisXRot = (int)vJoyCenter;
            joystickState.AxisYRot = (int)vJoyCenter;
            joystickState.AxisZRot = (int)vJoyCenter;
            joystickState.Slider = (int)vJoyMin;
            joystickState.Dial = (int)vJoyMin;
            joystickState.bHats = 36001;
        }


        //
        // Start feeder
        //
        public void Start()
        {
            lock (_lock)
            {
                if (!_running)
                {
                    try
                    {
                        _running = true;

                        if (!joystick.vJoyEnabled())
                        {
                            throw new Exception("vJoy is not enabled!");
                        }

                        VjdStat status = joystick.GetVJDStatus(joystickIndex);
                        // Acquire the target
                        if ((status == VjdStat.VJD_STAT_OWN) || ((status == VjdStat.VJD_STAT_FREE) && (!joystick.AcquireVJD(joystickIndex))))
                        {
                            _running = false;
                            throw new Exception("Failed to acquire vJoy device number " + joystickIndex);
                        }

                        long min, max;
                        min = 0;
                        max = 0;
                        joystick.GetVJDAxisMin(joystickIndex, HID_USAGES.HID_USAGE_X, ref min);
                        joystick.GetVJDAxisMax(joystickIndex, HID_USAGES.HID_USAGE_X, ref max);

                        vJoyMin = min;
                        vJoyMax = max;
                        vJoyCenter = vJoyMin + (vJoyMax - vJoyMin) / 2;

                        // VJoy reset to default
                        joystick.ResetVJD(joystickIndex);
                        ResetJoystickState();
                        joystick.UpdateVJD(joystickIndex, ref joystickState);

                        _thread = new Thread(new ThreadStart(this.Run));
                        _thread.Start();

                    }
                    catch (Exception ex)
                    {
                        _running = false;
                        throw ex;
                    }
                }
            }
        }


        //
        // Stop feeder
        //
        public void Stop()
        {
            if (!_running || _thread == null)
                return;

            _running = false;
            _thread.Join();
        }


        private void Run()
        {
            State controllerState;
            Controller controller = null;

            double setupCounter = 0.0;

            while (true)
            {
                // Stop?
                lock (_lock)
                {
                    if (!_running)
                    {
                        break;
                    }
                }


                if (config.Controller >= 1 && config.Controller <= 4)
                {
                    controller = controllers[config.Controller - 1];
                }

                if (controller != null && controller.IsConnected)
                {
                    controllerState = controller.GetState();
                    output = (double)controllerState.Gamepad.LeftThumbX / 32768.0;
                    output = Tools.CenterDeadzone(output, config.SteeringDeadzone);
                    output = Tools.Gamma(output, config.SteeringGamma);
                    output *= config.SteeringLockLimit;
                }
               

                if(setupModeLeft)
                    output = Math.Sin(setupCounter) * 0.5 - 0.5;
                if (setupModeRight)
                    output = Math.Sin(setupCounter) * 0.5 + 0.5;

                setupCounter += (0.20 / (double)SLEEPTIME);

                if (output > 1.0)
                    output = 1.0;
                if (output < -1.0)
                    output = -1.0;

                joystickState.AxisX = (int)Math.Round(output * vJoyCenter + vJoyCenter);

                // Update joystick state to virtual device
                joystick.UpdateVJD(joystickIndex, ref joystickState);

                // Wait
                Thread.Sleep(SLEEPTIME);
            }

            ResetJoystickState();
            joystick.UpdateVJD(joystickIndex, ref joystickState);
            joystick.RelinquishVJD(joystickIndex);

        }
    }
}
